#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "non_gui_defs.h"
#include "breeding.h"

/* generate 1 to 6 pups using the speed and stamina codes and the
   ancestor_coefficients of the parents; 'pups' has to be an array
   of size 6. */
gint mating(dog father, dog mother, dog *pups, gint number_of_pups)
{
    gint i;

    /* generate the values for the pups */
    for(i=0;i<number_of_pups;i++)
    {
	/* fill in name, gender etc.
	   level is 0 and 'startup' is 0 */
	fill_in_general_data(&pups[i],names_kept,0,0);
	
	/* the id value has to be filled in after the
	 human player decides to keep the pup */
	pups[i].id = -1;

	pups[i].mother_id=mother.id;
	pups[i].father_id=father.id;

	strcpy(pups[i].mother_name, mother.name);
	strcpy(pups[i].father_name, father.name);
	
	make_up_last_races(&pups[i]);
	
	/* calculate pup's speed and stamina */
	generate_pup_skill(father.speed, mother.speed, father.ancestor_speed,
			   mother.ancestor_speed, pups[i].speed,
			   kinship_of(father, mother));
	generate_pup_skill(father.stamina, mother.stamina,
			   father.ancestor_stamina,
			   mother.ancestor_stamina, pups[i].stamina,
			   kinship_of(father, mother));

	/* calculate pup's heritage */
	generate_ancestor_skill(father.speed, mother.speed,
				father.ancestor_speed,
				mother.ancestor_speed, pups[i].ancestor_speed);
	generate_ancestor_skill(father.stamina, mother.stamina,
				father.ancestor_stamina,
				mother.ancestor_stamina,
				pups[i].ancestor_stamina);
    }

    for(i=number_of_pups;i<6;i++)
	pups[i].age = -1;

    return 0;
}

/* find out if the mating has been succesful and write the
   results into the dogs' status.
   type = 0: it's a mating with equal rights 
   type = 1: it's a disadvantageous mating for the human player */
gint mate(dog *Dog1, dog *Dog2, gint type)
{
    /* the mating may not have been successful */
    if(rnd(0,1) < Dog1->fertility && rnd(0,1) < Dog1->fertility &&
       rnd(0,1) < 0.95)
    {
	Dog1->status =
	    (Dog1->gender == 1) ?
	    type * 5000 + 4100 + Dog2->id : 18;
	Dog2->status =
	    (Dog2->gender == 1) ?
	    type * 5000 + 4100 + Dog1->id : 18;
    }
    /* 4001 means a faked pregnancy */
    else
    {
	Dog1->status =
	    (Dog1->gender == 1) ?
	    4001 : Dog1->status;
	Dog2->status =
	    (Dog2->gender == 1) ?
	    4001 : Dog2->status;
	
    }

    return 0;
}

/* calculate a pup's skill from four 0-1-arrays that signify its ancestors */
gint generate_pup_skill(gint *father_skill, gint *mother_skill,
		       gint *father_ancestor_skill, gint *mother_ancestor_skill,
		       gint *pup_skill, gint kinship_parents)
{
    gfloat separator1, separator2, random;
    gfloat defect_probability = -1;
    gint i;

    /* separator1 is the probability that given an index between
       0 and 98 the array element corresponding to that index in
       the father_ancestor_skill array is chosen as element in the
       pup's array.
       0,25 - separator1 is the probability that mother_ancestor_skill
       is chosen.
       separator2 - 0,25 is the probability that father_skill is chosen.
       1 - separator2 is the probability that mother_skill is chosen.
       thus, the pup inherits a 0 or a 1 from its parents with probability
       0.75 and from its other ancestors 0,25 */
    separator1=gaussvt(0,0,0.25,0.25);
    separator2=gaussvt(0.25,0.25,1,1);

    /* find out whether there's incest */
    /* siblings or parent/child */
    if( kinship_parents == 1)
    {
	if(rnd(0,1) < 0.7)
	    defect_probability = gaussvt(0.1, 0.15, 0.30, 0.35);
    }
    /* half-siblings */
    else if(kinship_parents == 2)
    {
	if(rnd(0,1) < 0.5)
	    defect_probability = gaussvt(0.05, 0.1, 0.2, 0.25);
    }

    /* now for each array element we generate a random number and determine
       from whom the value of the array element is inherited */
    for(i=0;i<99;i++)
    {
	random=rnd(0,1);
	if(random <= separator1)
	    pup_skill[i] = father_ancestor_skill[i];
	else if(random <= 0.25)
	    pup_skill[i] = mother_ancestor_skill[i];
	else if(random <= separator2)
	    pup_skill[i] = father_skill[i];
	else
	    pup_skill[i] = mother_skill[i];

	/* defect? */
	if(rnd(0,1) < defect_probability)
	    pup_skill[i] = 0;
    }

    return 0;
}

/* put together piecewise the four ancestor arrays to calculate
   a new ancestor skill */
gint generate_ancestor_skill(gint *father_skill, gint *mother_skill,
			    gint *father_ancestor_skill,
			    gint *mother_ancestor_skill,
			    gint *pup_ancestor_skill)
{
    /* as above these figures determine how large a part is
       inherited from each array:
       separator1 elements are from father_ancestor_skill,
       50 - separator1 are from mother_ancestor_skill,
       separator2 - 50 are from father_skill
       and 99 - separator2 from mother_skill */
    gint separator1, separator2, i;
    gint order[99];

    /* with this permutation we avoid the situation that the first few
       elements of the new array are always pasted from the beginning of
       father_ancestor_skill and the last few from the end of mother_skill
       and so on */
    write_permutation(order,0,98);
    
    separator1=(gint)rint(rnd(0,49));
    separator2=(gint)rint(rnd(50,99));

    for(i=0;i<separator1;i++)
	pup_ancestor_skill[i]=father_ancestor_skill[order[i]];
    for(i=separator1;i<50;i++)
	pup_ancestor_skill[i]=mother_ancestor_skill[order[i]];
    for(i=50;i<separator2;i++)
	pup_ancestor_skill[i]=father_skill[order[i]];
    for(i=separator2;i<99;i++)
	pup_ancestor_skill[i]=mother_skill[order[i]];
    
    return 0;
}

/* find out whether and how two dogs are related */
gint kinship_of(dog Dog1, dog Dog2)
{
    /* do they have parents in the game? */
    if( strcmp(Dog1.father_name, "NAME") != 0 ||
	strcmp(Dog2.father_name, "NAME") != 0 ||
	strcmp(Dog1.mother_name, "NAME") != 0 ||
	strcmp(Dog2.mother_name, "NAME") != 0  )
    {

	/* siblings */
	if( strcmp(Dog1.father_name, Dog2.father_name) == 0 &&
	    strcmp(Dog1.father_name, Dog2.mother_name) == 0 )
	    return 1;
	
	/* parent */
	else if( strcmp(Dog1.name, Dog2.father_name) == 0 ||
		 strcmp(Dog1.name, Dog2.mother_name) == 0 ||
		 strcmp(Dog2.name, Dog1.father_name) == 0 ||
		 strcmp(Dog2.name, Dog1.mother_name) == 0 )
	    return 1;
	
	/* half-siblings */
	else if( strcmp(Dog1.father_name, Dog2.father_name) == 0 ||
		 strcmp(Dog1.mother_name, Dog2.mother_name) == 0 )
	    return 2;
    }

    else 
	return 0;

    return 0;
}
