#include <unistd.h>
#include <stdio.h>

#include "non_gui_defs.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "gui_defs.h"


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Callbacks for other windows than the main window ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Callbacks for the help window ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void
on_help_close_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *help_window = lookup_widget(GTK_WIDGET(button),
					   "help_window");

    gtk_widget_destroy(help_window);
}



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Callbacks for the ok_or_cancel window ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* a confirmation window that pops up whenever there's
   something important to confirm like giving away dogs */
void
on_ok_or_cancel_ok_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *ok_or_cancel_window = lookup_widget(GTK_WIDGET(button),
						"ok_or_cancel_window");
    GtkWidget *ok_or_cancel_type = lookup_widget(GTK_WIDGET(button),
					      "ok_or_cancel_type");
    GtkWidget *mate_dogs = lookup_widget(main_window, "dog_buttons_mate_dogs");
    GtkWidget *menu_item = lookup_widget(main_window, "menu_new_game");
    
    gint type = get_int_from_label(GTK_LABEL(ok_or_cancel_type));
    gint dog_indices[2];

    /* giving away a dog */
    if(type == 0)
	on_dog_buttons_give_away_clicked(GTK_BUTTON(mate_dogs),
					 GINT_TO_POINTER(1));
    /* proceeding to race results and next
       race weekend: call go_to_races with
       override */
    else if(type == 1)
	on_go_to_races_clicked(GTK_BUTTON(mate_dogs),
			       GINT_TO_POINTER(1));
    /* disadvantageous mating */
    else if(type == 2)
    {
	get_dog_indices(main_window, dog_indices);
	
	mate(&dogs[dog_indices[0]], &dogs[dog_indices[1]], 1);
	display_kennel(main_window, 1, 0, 10);
	display_kennel(main_window,
			(gint)((gfloat)dog_indices[1] / 20) + 1, 1, 10);

	set_save_status(main_window, 0);
    }
    /* incest: call mate_dogs with the incest override argument */
    else if(type == 3)
	on_mate_dogs_clicked(GTK_BUTTON(mate_dogs), GINT_TO_POINTER(1));
    /* start new game */
    else if(type == 10)
	on_menu_new_game_activate(GTK_MENU_ITEM(menu_item),
				  GINT_TO_POINTER(1));
    /* quit without saving */
    else if(type == 50)
	on_toolbar_quit_clicked(GTK_BUTTON(mate_dogs),
				GINT_TO_POINTER(1));
    /* question at the beginning of the game whether or not
       to load a saved game */
    else if(type == 100)
	on_toolbar_load_clicked(GTK_BUTTON(mate_dogs),
				GINT_TO_POINTER(1));

    change_popups_active(main_window, -1);
    set_buttons(main_window);
    gtk_widget_destroy(ok_or_cancel_window);
}
 
void
on_ok_or_cancel_alternative_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *ok_or_cancel_window = lookup_widget(GTK_WIDGET(button),
						"ok_or_cancel_window");
    GtkWidget *ok_or_cancel_type = lookup_widget(GTK_WIDGET(button),
						 "ok_or_cancel_type");
    GtkWidget *toolbar_save = lookup_widget(main_window,
					    "toolbar_save");
    gint type = get_int_from_label(GTK_LABEL(ok_or_cancel_type));

    if(type == 50)
    {
	on_toolbar_save_clicked(GTK_BUTTON(toolbar_save),
				GINT_TO_POINTER(1));
	gtk_widget_destroy(ok_or_cancel_window);
    }
    else if(type == 100)
    {
	gtk_widget_destroy(ok_or_cancel_window);
	start(main_window, 0);
    }

    change_popups_active(main_window, -1);    
}

void
on_ok_or_cancel_cancel_clicked                (GtkButton       *button,
					       gpointer         user_data)
{
    GtkWidget *ok_or_cancel_window = lookup_widget(GTK_WIDGET(button),
						"ok_or_cancel_window");
    GtkWidget *ok_or_cancel_type = lookup_widget(GTK_WIDGET(button),
						 "ok_or_cancel_type");
    gint type = get_int_from_label(GTK_LABEL(ok_or_cancel_type));
    
    change_popups_active(main_window, -1);
    
    if(type != 100)
	gtk_widget_destroy(ok_or_cancel_window);    
    else
	gtk_main_quit();
}


/* change confirmation options */
void
on_ok_or_cancel_check_button_toggled      (GtkToggleButton *togglebutton,
					   gpointer         user_data)
{
    GtkWidget *ok_or_cancel_type = lookup_widget(GTK_WIDGET(togglebutton),
					      "ok_or_cancel_type");
    gint type = get_int_from_label(GTK_LABEL(ok_or_cancel_type));
    gint i;
    
    for(i=0;i<12;i++)
	if(type == i)
	    options_confirm[i] = (options_confirm[i] + 1 ) % 2;
}



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Callbacks for the news window ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void
on_news_window_ok_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *news_window = lookup_widget(GTK_WIDGET(button),
					   "news_window");
    GtkWidget *news_window_type = 
	lookup_widget(GTK_WIDGET(button),"news_window_type");
    GtkWidget *toolbar_load = 
	lookup_widget(main_window, "toolbar_load");
    gint type = get_int_from_label(GTK_LABEL(news_window_type));

    /* decrement number of active popups */
    change_popups_active(main_window, -1);
    
    gtk_widget_destroy(news_window);
    
    /* if we load at the beginning of the game via command-line parameter,
       and fail to open the file, show file selection */
    if(type == 200)
	on_toolbar_load_clicked(GTK_BUTTON(toolbar_load),
				GINT_TO_POINTER(1));
}

void
on_news_window_checkbutton_toggled     (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    GtkWidget *news_window = lookup_widget(GTK_WIDGET(togglebutton),
					   "news_window");
    GtkWidget *news_window_type =
	lookup_widget(news_window, "news_window_type");
    gint type = get_int_from_label(GTK_LABEL(news_window_type));
    gint i;
    
    for(i=0;i<7;i++)
	if(type == i)
	    options_show_news[i] = (options_show_news[i] + 1 ) % 2;

    if(type == 7)
    {
	if(options_misc[18] < 100)
	    options_misc[18] += 100;
	if(options_misc[19] < 100)
	    options_misc[19] += 100;
    }	

    set_save_status(main_window, 0);
}



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Callbacks for the ask_name window ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void
on_ask_name_yes_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *user_name;
    GtkWidget *ask_name_window;
    GtkWidget *ask_name_entry;

    /* we need to destroy the window afterwards */
    ask_name_window = lookup_widget(GTK_WIDGET(button), "ask_name_window");

    /* get entry from the entry window */
    ask_name_entry = lookup_widget(GTK_WIDGET(button), "ask_name_entry");
    if(strlen(gtk_entry_get_text(GTK_ENTRY(ask_name_entry))) > 0)
	strcpy(kennels[0].name, gtk_entry_get_text(GTK_ENTRY(ask_name_entry)));
    else
	strcpy(kennels[0].name, "NONAME");

    /* find the appropriate place for it in the main window */
    user_name = lookup_widget(main_window, "current_kennel_owner_name");

    gtk_label_set_text( GTK_LABEL(user_name), kennels[0].name);

    gtk_widget_destroy(GTK_WIDGET(ask_name_window));

    /* show main window */
    gtk_widget_show(main_window);
}

void
on_ask_name_entry_activate             (GtkEditable     *editable,
                                        gpointer         user_data)
{
    GtkWidget *button = lookup_widget(GTK_WIDGET(editable), "ask_name_yes");
    
    on_ask_name_yes_clicked(GTK_BUTTON(button), NULL);
}



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Callbacks for the options window ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void
on_options_ok_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *options_window = lookup_widget(GTK_WIDGET(button),
					      "options_window");

    set_buttons_options_window(GTK_WIDGET(button), 10);

    set_save_status(main_window, 0);
    change_popups_active(main_window, -1);
    set_buttons(main_window);
    gtk_widget_destroy(options_window);
}

void
on_options_cancel_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *options_window = lookup_widget(GTK_WIDGET(button),
					      "options_window");
    
    change_popups_active(main_window, -1);
    gtk_widget_destroy(options_window);
}

void
on_options_misc18_pressed              (GtkButton       *button,
                                        gpointer         user_data)
{
    on_options_misc_toggled(GTK_TOGGLE_BUTTON(button), 
			    GINT_TO_POINTER(18));
}

void
on_options_misc19_pressed              (GtkButton       *button,
                                        gpointer         user_data)
{
    on_options_misc_toggled(GTK_TOGGLE_BUTTON(button), 
			    GINT_TO_POINTER(19));
}

void
on_options_misc_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    gint number = GPOINTER_TO_INT(user_data);

    if(gtk_toggle_button_get_active(togglebutton))
	options_misc[number] = options_misc[number] % 100;
    else
	options_misc[number] = options_misc[number] % 100 + 100;

    set_buttons_options_window(GTK_WIDGET(togglebutton), 0);
}



void
on_options_misc0_0_toggled             (GtkButton       *button,
                                        gpointer         user_data)
{
    on_options_misc0_toggled(button, GINT_TO_POINTER(0));
}


void
on_options_misc0_1_toggled             (GtkButton       *button,
                                        gpointer         user_data)
{
    on_options_misc0_toggled(button, GINT_TO_POINTER(1));
}


void
on_options_misc0_2_toggled             (GtkButton       *button,
                                        gpointer         user_data)
{
    on_options_misc0_toggled(button, GINT_TO_POINTER(2));
}

void
on_options_misc0_toggled (GtkButton *button,
			    gpointer user_data)
{
    gint value = GPOINTER_TO_INT(user_data);

    if(value == 0)
	options_misc[0] = options_misc[0] % 100;
    else if(value == 1)
	options_misc[0] = options_misc[0] % 100 + 100;
    else if(value == 2)
	options_misc[0] = options_misc[0] % 100 + 200;

    set_buttons_options_window(GTK_WIDGET(button), 0);
}



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Callbacks for the file selection window ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


void
on_file_selection_ok_clicked   (GtkButton       *button,
				gpointer         user_data)
{
    GtkWidget *file_selection = lookup_widget(GTK_WIDGET(button),
					      "file_selection");
    GtkWidget *toolbar_quit = lookup_widget(main_window,
					    "toolbar_quit");
    GtkWidget *gui_info = lookup_widget(main_window, "gui_info");
    gchar *temporary_gchar;
    gchar *ok_button_text;
    FILE *test;
    gint done = 0;
    int type = GPOINTER_TO_INT(user_data);
    int info = get_int_from_label(GTK_LABEL(gui_info));
    gint news_factor = (type != 5) ? 1 : 2;
    
    namescan(&temporary_gchar,
	     gtk_file_selection_get_filename(
		 GTK_FILE_SELECTION(file_selection)));
    
    if(info == 0 || info == 100)
    {
	save_game(temporary_gchar);
	if(info == 100)
	    on_toolbar_quit_clicked(GTK_BUTTON(toolbar_quit),
				    NULL);
	done = 1;
    }
    else
    {
	
	if((test = fopen(temporary_gchar, "rb")) == NULL)
	    print_news(main_window, "Could not open file.",
		       100 * news_factor);
	else
	{
	    fclose(test);
	    if(load_game(temporary_gchar, 0) == 5)
		print_news(main_window, "Does not seem to be a Greyhounds saved game.", 100 * news_factor);
	    else
		done = 1;
	}
    }

    if(done == 1)
    {
	on_back_to_main_clicked(GTK_BUTTON(toolbar_quit), NULL);

	set_save_status(main_window, 1);
	write_labels(main_window);
	set_buttons(main_window);
	change_popups_active(main_window, -1);
	
	/* show main window if this is loading at the
	   beginning of the game */
	if(info == 250)
	    gtk_widget_show(main_window);
	
	gtk_widget_destroy(file_selection);
    }
    else
	if(type == 5)
	{
	    change_popups_active(main_window, -1);
	    gtk_widget_destroy(file_selection);
	}
}

void
on_file_selection_cancel_clicked (GtkButton       *button,
					  gpointer         user_data)
{    
    GtkWidget *file_selection = lookup_widget(GTK_WIDGET(button),
					      "file_selection");
    GtkWidget *gui_info = lookup_widget(main_window, "gui_info");
    int info = get_int_from_label(GTK_LABEL(gui_info));
    
    change_popups_active(main_window, -1);

    if(info != 500)
	gtk_widget_destroy(file_selection);
    else
	gtk_main_quit();
}



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Callbacks for the live race window ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void
on_live_race_cancel_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *live_race_label = lookup_widget(GTK_WIDGET(button),
					       "live_race_label");

    gtk_label_set_text(GTK_LABEL(live_race_label), "-1");    
}

void
on_live_race_close_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *live_race_window = lookup_widget(GTK_WIDGET(button),
						"live_race_window");
    
    change_popups_active(main_window, -1);
    gtk_widget_destroy(live_race_window);
}


void
on_live_race_next_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *live_race_window = lookup_widget(GTK_WIDGET(button),
						"live_race_window");
    GtkWidget *live_race_label = lookup_widget(GTK_WIDGET(button),
					       "live_race_label");
    GtkWidget *race_number = lookup_widget(GTK_WIDGET(button),
					   "race_number");
    GtkWidget *race_level = lookup_widget(GTK_WIDGET(button),
					  "race_level");
    gint race_nr = get_int_from_label(GTK_LABEL(race_number));
    gint next_race = next_user_race_levels(race_nr);

    if(get_int_from_label(GTK_LABEL(live_race_label)) == -1)
    {
	change_popups_active(main_window, -1);
	gtk_widget_destroy(live_race_window);
    }
    else
	gtk_label_set_text(GTK_LABEL(live_race_label), "-1");
    
    show_live_race(NULL, next_race);
}



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* ### Callbacks for the main window ### */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* sort list of dogs in the usual way when a column title is clicked */
void
on_dog_list_click_column               (GtkCList        *clist,
                                        gint             column,
                                        gpointer         user_data)
{
    GtkWidget *cpu_kennel = lookup_widget(GTK_WIDGET(clist),
					  "dog_info_label_2");
    GtkWidget *dog_info_label = lookup_widget(GTK_WIDGET(clist),
					      "dog_info_label");
    GtkWidget *current_kennel_id =
	lookup_widget(GTK_WIDGET(clist), "current_kennel_id");

    gint i;
    gint kennel_id = get_int_from_label(GTK_LABEL(current_kennel_id));
    gint cpu_kennel_id = get_int_from_label(GTK_LABEL(cpu_kennel));
    gchar *label;

    gtk_label_get(GTK_LABEL(dog_info_label), &label);
    
    /* we refuse to sort by name */
    if(column == 0)
    {
	print_message( GTK_WIDGET(clist),
		       "C'mon, makes no sense sorting by that.");
    }
    else
    {
	/* kennel id on screen is 1 to 20 contrary
	   to the intern 0 to 19 */
	for(i=0;i<21;i++)
	    sort_dogs(i, column);

	/* if in dog_info a cpu-kennel is displayed,
	   update it too */
	if( strcmp(label, " Trading with  Kennel: ") == 0 )
	{
	    display_kennel(GTK_WIDGET(clist), 1, 0, 20);
	    display_kennel(GTK_WIDGET(clist), cpu_kennel_id, 1, 20);
	}
	else if( strcmp(label, " Mating dogs  Kennel: ") == 0 )
	{
	    display_kennel(GTK_WIDGET(clist), 1, 0, 10);
	    display_kennel(GTK_WIDGET(clist), cpu_kennel_id, 1, 10);
	}
	else
	    display_kennel(GTK_WIDGET(clist), kennel_id, 0, -1);
    }
    
    set_buttons(GTK_WIDGET(clist));
}


/* quite multifunctional buttons, 'forward' and back'.
   they always turn a page of dog_list or dog_info */

void
on_kennel_left_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *gui_info = lookup_widget(GTK_WIDGET(button),
					"gui_info");
    
    gtk_label_set_text(GTK_LABEL(gui_info), "-1");
    on_kennel_left_or_right_clicked(button, GINT_TO_POINTER(-1));
    gtk_label_set_text(GTK_LABEL(gui_info), "0");
}


void
on_kennel_right_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *gui_info = lookup_widget(GTK_WIDGET(button),
					"gui_info");
    
    gtk_label_set_text(GTK_LABEL(gui_info), "1");
    on_kennel_left_or_right_clicked(button, GINT_TO_POINTER(1));
    gtk_label_set_text(GTK_LABEL(gui_info), "0");
}

void
on_kennel_left_or_right_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dog_list = lookup_widget(GTK_WIDGET(button), "dog_list");
    GtkWidget *selected_row_dog_list = 
	lookup_widget(GTK_WIDGET(button), "selected_row_dog_list");
    GtkWidget *current_kennel_id =
	lookup_widget(GTK_WIDGET(button), "current_kennel_id");
    GtkWidget *dog_info_label
	= lookup_widget(GTK_WIDGET(button), "dog_info_label");
    GtkWidget *dog_info_label_2
	= lookup_widget(GTK_WIDGET(button), "dog_info_label_2");
    GtkWidget *race_number = lookup_widget(GTK_WIDGET(button),
					   "dog_info_label_6");
    GtkWidget *cpu_kennel = lookup_widget(GTK_WIDGET(button),
					  "dog_info_label_2");

    gchar *info_label;
    gchar *info_label_2;
    gint number;
    gint kennel_id;
    gint race_type;
    gint left_or_right = GPOINTER_TO_INT(user_data);
    gint row = get_int_from_label(GTK_LABEL(selected_row_dog_list));

    gtk_label_get(GTK_LABEL(dog_info_label), &info_label);
    gtk_label_get(GTK_LABEL(dog_info_label_2), &info_label_2);
    
    /* if the human player's only browsing through the kennels */
    if(strcmp(info_label, " Detailed dog information") == 0)
    {
	kennel_id = get_int_from_label(GTK_LABEL(current_kennel_id));
	display_kennel(GTK_WIDGET(button),
		       kennel_id + left_or_right, 0, 0);
    }
    /* else he's browsing through the cpu-kennels for
       trading  */
    else if(strcmp(info_label, " Trading with  Kennel: ") == 0)
    {
	kennel_id = get_int_from_label(GTK_LABEL(cpu_kennel));
	
	display_kennel(GTK_WIDGET(button), kennel_id + left_or_right, 1, 20);
    }
    /* ...or breeding */
    else if(strcmp(info_label, " Mating dogs  Kennel: ") == 0)
    {
	kennel_id = get_int_from_label(GTK_LABEL(cpu_kennel));	
	display_kennel(GTK_WIDGET(button),
		       next_kennel_in_heat(kennel_id, left_or_right), 1, 10);
	/* select once again the user's dog to filter out the
	   dogs in dog_info that can't be mated */
	on_dog_list_select_row(GTK_CLIST(dog_list),
			       row, 0, NULL, NULL);
    }
    /* else he's browsing through the race lists */
    else if(strcmp(info_label, " Entering dogs  Level: ") == 0 ||
	    strcmp(info_label, " Race results  Level: ") == 0 ||
	    strcmp(info_label, " Last races  Level: ") == 0)
    {
	/* find out number and type of race */
	number = get_int_from_label(GTK_LABEL(race_number));
	if(strncmp(info_label, " Ent", 3) == 0)
	    race_type = 1;
	else if(strncmp(info_label, " Rac", 3) == 0)
	    race_type = 0;
	else
	    race_type = 2;
	
	if(strncmp(info_label_2, "any", 2) != 0)
	    race_type += 30;
	else
	    race_type += 40;
	
	display_races(GTK_WIDGET(button),
		      next_user_race(number, race_type, left_or_right) % 100,
		      race_type);
    }
    /* looking at statistics */
    else if(strncmp(info_label, " Statistics  Dogs on", 17) == 0)
	show_statistics(GTK_WIDGET(button), 0, left_or_right, 20);
    else if(strncmp(info_label, " Statistics  Kennels", 17) == 0)
	show_statistics(GTK_WIDGET(button), 0, left_or_right, 10);
    else if(strncmp(info_label, " Statistics", 6) == 0 &&
	    strncmp(info_label_2, "Hist", 3) == 0)
	show_history(GTK_WIDGET(button), left_or_right);
    
    /* change button sensitivity */
    set_buttons(GTK_WIDGET(button));
}

/* switch back to the user's kennel */
void
on_toolbar_home_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    on_home_kennel_clicked(button, GINT_TO_POINTER(1));
}

/* switch back to the main screen with either the home
   kennel or the one currently displayed */
void
on_home_kennel_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *current_kennel = lookup_widget(GTK_WIDGET(button),
					      "current_kennel_id");
    gint current_kennel_id = get_int_from_label(GTK_LABEL(current_kennel));
    gint data = GPOINTER_TO_INT(user_data);

    if(data == 1)
	current_kennel_id = 1;

    display_kennel(GTK_WIDGET(button), 
		   current_kennel_id, 0, 0);
    
    set_buttons(GTK_WIDGET(button));
}

/* prgint detailed information about a dog if the human player
   clicks on its row in dog_list */
void
on_dog_list_select_row                 (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    GtkWidget *dog_info	= lookup_widget( GTK_WIDGET(clist), "dog_info");;
    GtkWidget *selected_row = 
	lookup_widget(GTK_WIDGET(clist), "selected_row_dog_list");
    GtkWidget *dog_info_label	= lookup_widget( GTK_WIDGET(clist),
						 "dog_info_label");;
    GtkWidget *race_number = lookup_widget(GTK_WIDGET(clist),
					   "dog_info_label_6");
    GtkWidget *cpu_kennel = lookup_widget(GTK_WIDGET(clist),
					  "dog_info_label_2");
    GtkWidget *enter_button = lookup_widget(GTK_WIDGET(clist),
					    "dog_buttons_enter_dog");
    gchar *info_label;
    gint cpu_kennel_id = get_int_from_label(GTK_LABEL(cpu_kennel));
    gint rnumber = get_int_from_label(GTK_LABEL(race_number)) - 1;
    gint dog_indices[2];
    gint old_row = get_int_from_label(GTK_LABEL(selected_row));
    gint data = GPOINTER_TO_INT(user_data);

        
    gtk_label_get(GTK_LABEL(dog_info_label), &info_label);

    gtk_clist_unselect_row(clist, old_row, 0);
    gtk_clist_unselect_row(clist, row, 0);

    write_selected_row(GTK_WIDGET(clist), row, 0);

    get_dog_indices(GTK_WIDGET(clist), dog_indices);

    /* change colors of (un)selected rows
       and injured dogs and bitches in heat.
       if column is 1 the calling function was
       'enter_dogs_in_races' and we omit this,
       otherwise selecting the fitting dog and race
       is too slow */
    if(column != 1)
	change_colors_clist(clist, 0);

    /* if the user's entering dogs in races,
       we write additional information in shorter form
       into the message window, else into the dog_info
       window */
    if(strcmp(info_label, " Detailed dog information") == 0)
	print_dog_info( dogs[ dog_indices[0] ],
			GTK_CLIST(dog_info) );
    else if(strcmp(info_label, " Entering dogs  Level: ") == 0)
    {
	print_short_dog_info(GTK_WIDGET(clist),
			     rnumber, dogs[ dog_indices[0] ] );
	if(/* row == old_row */
	    event != NULL &&
	    event->type == GDK_2BUTTON_PRESS &&
	    set_sensitive_enter_dogs(dog_indices[0], rnumber) )
	    on_enter_dog_in_race_clicked(
		GTK_BUTTON(enter_button), NULL);
    }

    /* if this is mating dogs, we sort out the
       dogs on the other list that cannot be mated
       with the selected one */
    else if(strcmp(info_label, " Mating dogs  Kennel: ") == 0)
    {
	if(dogs[dog_indices[0]].gender == 0 &&
	   dogs[dog_indices[0]].status < 20 &&
	   kennel_has_bitch_in_heat(cpu_kennel_id - 1) == 1)
	    display_kennel(GTK_WIDGET(clist), cpu_kennel_id, 1, 12);
	else if(dogs[dog_indices[0]].status == 0 &&
		dogs[dog_indices[0]].age > 11)
	    display_kennel(GTK_WIDGET(clist), cpu_kennel_id, 1, 11);
	else
	    display_kennel(GTK_WIDGET(clist), cpu_kennel_id, 1, 10);
    }

    set_buttons(GTK_WIDGET(clist));
}

/* if the user moves to another dog in the dog list
   we move the selection, too */
void
on_dog_list_scroll_vertical            (GtkCList        *clist,
                                        GtkScrollType    scroll_type,
                                        gfloat           position,
                                        gpointer         user_data)
{
    gint kennel_id;
    gint selected_row = get_selected_row( GTK_WIDGET(clist) , 0 );
    GtkWidget *current_kennel_id =
	lookup_widget(GTK_WIDGET(clist), "current_kennel_id");
    
    kennel_id = get_int_from_label(GTK_LABEL(current_kennel_id)) - 1;

    /* scrolling down if possible */
    if((gint)scroll_type == 3 && selected_row < 
       dogs_in_kennel(kennel_id) - 1)
	on_dog_list_select_row(clist, selected_row + 1,
			       0, NULL, NULL );

    /* scrolling up if possible */
    else if((gint)scroll_type == 2 && selected_row > 0)
	on_dog_list_select_row(clist, selected_row - 1,
			       0, NULL, NULL );
}

/* remove a dog from the list */
void
on_dog_buttons_give_away_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *current_kennel_id =
	lookup_widget(GTK_WIDGET(button), "current_kennel_id");
    gint override = GPOINTER_TO_INT(user_data);
    gint id = get_int_from_label(GTK_LABEL(current_kennel_id));
    
    /* if give_away_dog confirmation is on,
       ask user if he's sure */
    if( options_confirm[0] == 1 && override != 1)
	show_ok_or_cancel_window(GTK_WIDGET(button), " Are you sure you want to give away the poor creature? ", 0);
    else
    {
	remove_dog(GTK_WIDGET(button));
	display_kennel(GTK_WIDGET(button), id, 0, 0);
	set_save_status(GTK_WIDGET(button), 0);
	set_buttons(GTK_WIDGET(button));
    }
}

void
on_enter_dogs_in_races_button_clicked  (GtkButton       *button,
                                        gpointer         user_data)
{
    gint i,j;
    gint dog_indices[2];
    gint displayed = 0;
    GtkWidget *dog_list = lookup_widget( GTK_WIDGET(button), "dog_list");
    GtkWidget *dog_info = lookup_widget( GTK_WIDGET(button), "dog_info");

    /* sort the dog list by level, ascending */
    for(i=0;i<20;i++)
	sort_dogs(i, 3);
    if(order[420] != -3)
	for(i=0;i<20;i++)
	    sort_dogs(i, 3);

    gtk_clist_freeze(GTK_CLIST(dog_list));
    gtk_clist_freeze(GTK_CLIST(dog_info));

    /* set the dog list back to kennel 1 */
    on_home_kennel_clicked(button, GINT_TO_POINTER(1));
    
    /* tell the user he's not qualified if he's not qualified :-) */
    if(month == 12 && week == 3 && 
       is_in_list(qualified_kennels, 0, 5, 0) == 0)
	print_message(GTK_WIDGET(button), "Sorry. You can't enter dogs, you're not qualified for the Trophy races.");

    if(dog_indices[0] != -1 && options_misc[1] == 1)
    {	
	/* pre-select the fitting dog and race if it's not
	   trophy racing */
	if(month != 12 || week != 3)
	{
	    for(i=0; i<12;i++)
	    {
		for(j=0; j<dogs_in_kennel(0); j++)
		{
		    on_dog_list_select_row(GTK_CLIST(dog_list),
					   j, 1, NULL, NULL);
		    get_dog_indices(GTK_WIDGET(button), dog_indices);
		    
		    if( races[1][i].dogs[9] == NULL &&
			set_sensitive_enter_dogs(dog_indices[0], i) )
		    {
			display_races(GTK_WIDGET(button), i + 1, 31);
			on_dog_list_select_row(GTK_CLIST(dog_list), j,
					       0, NULL, NULL);
			displayed = 1;
			break;
		    }
		}

		if( displayed == 1)
		    break;
	    }
	}
    }

    if( displayed == 0)
    {
	if( month != 12 || week != 3)
	    display_races(GTK_WIDGET(button), 1, 31);
	else
	    display_races(GTK_WIDGET(button), 1, 41);
    }

    gtk_clist_thaw(GTK_CLIST(dog_list));
    gtk_clist_thaw(GTK_CLIST(dog_info));

    set_buttons(GTK_WIDGET(button));
}

/* enter dog in a race */
void
on_enter_dog_in_race_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *race_number = lookup_widget(GTK_WIDGET(button),
					   "dog_info_label_6");
    GtkWidget *dog_list = lookup_widget(GTK_WIDGET(button), "dog_list");
    gint i,j;
    gint displayed = 0;
    gint selected_row;
    gint dog_indices[2];
    gint number;

    number = get_int_from_label(GTK_LABEL(race_number));
    selected_row = get_selected_row(GTK_WIDGET(button), 0);
    
    get_dog_indices(GTK_WIDGET(button), dog_indices);
    
    /* enter dog */
    for(i=0;i<10;i++)
	if(races[1][number - 1].dogs[i] == NULL ||
	   races[1][number - 1].dogs[i]->id < 20)
	{
	    races[1][number - 1].dogs[i] = &dogs[ dog_indices[0] ];
	    break;
	}

    /* scroll automatically to the next race and next dog that
       fit together if we don't have trophy races */
    gtk_clist_freeze(GTK_CLIST(dog_list));

    if((month != 12 || week != 3) && options_misc[1] == 1)
    {
	for(i=0; i<12;i++)
	{
	    for(j=0; j<dogs_in_kennel(0); j++)
	    {
		on_dog_list_select_row(GTK_CLIST(dog_list),
				       j, 1, NULL, NULL);
		get_dog_indices(GTK_WIDGET(button), dog_indices);
	    
		if( races[1][i].dogs[9] == NULL &&
		    set_sensitive_enter_dogs(dog_indices[0], i) )
		{
		    display_races(GTK_WIDGET(button), i + 1, 31);
		    on_dog_list_select_row(GTK_CLIST(dog_list),
					   j, 0, NULL, NULL);
		    gtk_clist_thaw(GTK_CLIST(dog_list));
		    displayed = 1;
		    break;
		}
	    }
	    
	    if( displayed == 1)
		break;
	}
    }

    if( displayed == 0)
    {
	display_races(GTK_WIDGET(button), number, 31);
	gtk_clist_thaw(GTK_CLIST(dog_list));
    }

    set_save_status(GTK_WIDGET(button), 0);    
    set_buttons(GTK_WIDGET(button));
}

void
on_back_to_main_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    on_home_kennel_clicked(button, NULL);
    
    /* show some popups with news if the news labels
       indicate something */
    news(GTK_WIDGET(button));
}


void
on_remove_dog_from_race_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *race_number = lookup_widget(GTK_WIDGET(button),
					   "dog_info_label_6");
    gint number = get_int_from_label(GTK_LABEL(race_number));
    gint i;
    
    for(i=0;i<10;i++)
	if(races[1][number-1].dogs[i] != NULL &&
	   races[1][number-1].dogs[i]->id < 20)
	{
	    races[1][number-1].dogs[i] = NULL;
	    display_races(GTK_WIDGET(button), number, 31);
	    break;
	}

    set_save_status(GTK_WIDGET(button), 0);	
    set_buttons(GTK_WIDGET(button));
}


void
on_go_to_races_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
    gint override = GPOINTER_TO_INT(user_data);

    /* if go_to_races confirmation is on,
       ask user if he's sure */
    if( options_confirm[1] == 1 && override != 1)
	show_ok_or_cancel_window(GTK_WIDGET(button), " Go to the races and end this week's activities? ", 1);
    else
    {	
	race_results();

	if(options_misc[0] < 200 && options_misc[0] > 99 ||
	   (options_misc[0] < 100 && next_user_race_levels(0) == -1))
	    show_progress(7, "Racing...");

	between_race_weekends(GTK_WIDGET(button));
	
	if(month != 1 || week != 1 || season == 1)
	{
	    if(next_user_race_levels(0) != -1 && options_misc[0] < 100)
		show_live_race(NULL, next_user_race_levels(0));
	    
	    display_races(GTK_WIDGET(button), 
			  next_user_race(12, 30, 1) % 100, 30);
	}
	else
	{
	    if(next_user_race_levels(0) != -1 && options_misc[0] < 100)
		show_live_race(NULL, next_user_race_levels(0));
	    
	    display_races(GTK_WIDGET(button), 
			  next_user_race(6, 40, 1) % 100, 40);
	}

	if(month != 12 || week != 3)
	    write_races();
	else
	    write_champions_trophy_races();

	set_buttons(GTK_WIDGET(button));
    }
}

/* swap two dogs if the cpu-kennel's owner accepts */
void
on_trade_dogs_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *cpu_kennel = lookup_widget(GTK_WIDGET(button),
					  "dog_info_label_2");
    gint cpu_kennel_id = get_int_from_label(GTK_LABEL(cpu_kennel));
    gint dog_indices[2];
    gint result;

    get_dog_indices(GTK_WIDGET(button), dog_indices);

    /* check whether the human player's not on the other
       kennel's black list (because of too many stupid offers) */
    if(options_kennel_block[1][ cpu_kennel_id - 1 ] > 2)
	print_message(GTK_WIDGET(button), "The kennel owner refuses to trade with you. He didn't like your offers in the past, it seems.");
    else
    {
	result = compare_dogs(dogs[dog_indices[0]],
			      dogs[dog_indices[1]], 1);
	if( result == 1 )
	{
	    swap_dogs(&dogs[dog_indices[0]], &dogs[dog_indices[1]]);
	    print_message(GTK_WIDGET(button), "The owner accepts your offer.");

	    /* update dog lists */
	    display_kennel(GTK_WIDGET(button), 1, 0, 0);
	    display_kennel(GTK_WIDGET(button), cpu_kennel_id, 1, 20);	    
	    set_save_status(GTK_WIDGET(button), 0);	
	}
	else if(result == 2)
	    print_message(GTK_WIDGET(button), "The owner rejects your offer.");
	else
	    print_message(GTK_WIDGET(button), "The owner rejects your offer and refuses to do any business with you until you get reasonable again.");
    }

    set_buttons(GTK_WIDGET(button));
}


/* the human player wants to trade dogs */
void
on_trading_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    /* show home kennel in dog_list and first cpu kennel
       in dog_info */
    display_kennel(GTK_WIDGET(button), 1, 0, 20);
    display_kennel(GTK_WIDGET(button), 2, 1, 20);
    
    set_buttons(GTK_WIDGET(button));
}


void
on_dog_info_select_row                 (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    write_selected_row(GTK_WIDGET(clist), row, 1);
    gtk_clist_unselect_row(clist, row, 0);

    /* change colors of (un)selected rows */
    change_colors_clist(clist, 1);

    set_buttons(GTK_WIDGET(clist));
}


void
on_dog_info_scroll_vertical            (GtkCList        *clist,
                                        GtkScrollType    scroll_type,
                                        gfloat           position,
                                        gpointer         user_data)
{
    gint kennel_id;
    gint selected_row = get_selected_row( GTK_WIDGET(clist) , 1 );
    GtkWidget *cpu_kennel_id =
	lookup_widget(GTK_WIDGET(clist), "dog_info_label_2");
    
    kennel_id = get_int_from_label(GTK_LABEL(cpu_kennel_id)) - 1;
    
    if(kennel_id < 20 && kennel_id > 0)
    {
	/* scrolling down if possible */
	if((gint)scroll_type == 3 && selected_row <
	   dogs_in_kennel(kennel_id) - 1)
	{
	    on_dog_info_select_row(clist, selected_row + 1,
				   0, NULL, NULL );
	    write_selected_row(GTK_WIDGET(clist), selected_row + 1, 1);
	}
	/* scrolling up if possible */
	else if((gint)scroll_type == 2 && selected_row > 0)
	{
	    on_dog_info_select_row(clist, selected_row - 1,
				   0, NULL, NULL );
	    write_selected_row(GTK_WIDGET(clist), selected_row - 1, 1);
	}
    }
}

/* show results of past semi-month's races */
void
on_race_results_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    if(season == 1 || month != 1 || week != 1 )
	display_races(GTK_WIDGET(button), next_user_race(12, 32, 1) % 100, 32);
    else
	display_races(GTK_WIDGET(button), next_user_race(6, 42, 1) % 100, 42);
    
    set_buttons(GTK_WIDGET(button));
}


/* the human player wants to mate two dogs */
void
on_breeding_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dog_list = lookup_widget(GTK_WIDGET(button), "dog_list");
    GtkWidget *selected_row =
	lookup_widget(GTK_WIDGET(button), "selected_row_dog_list");
    gint row = get_int_from_label(GTK_LABEL(selected_row));

    /* set the dog list window to home kennel */
    display_kennel(GTK_WIDGET(button), 1, 0, 10);
    
    /* check whether there are bitches in heat in the game */
    if(kennel_has_bitch_in_heat(0) == 0 &&
       next_kennel_in_heat(20,1) == 1)
    {
	print_message(GTK_WIDGET(button), "Sorry. There are currently no bitches in heat in the whole game.");
        on_back_to_main_clicked(button, NULL);
    }
    else
    {
	display_kennel(GTK_WIDGET(button),
		       next_kennel_in_heat(20,1), 1, 10);

	/* reselect the human player's dog to filter the dogs
	   on dog_info */
	on_dog_list_select_row(GTK_CLIST(dog_list), row, 0,
			       NULL, NULL);
    }
    
    set_buttons(GTK_WIDGET(button));
}

int
on_mate_dogs_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *cpu_kennel = lookup_widget(GTK_WIDGET(button),
					  "dog_info_label_2");
    
    gint cpu_kennel_id = get_int_from_label(GTK_LABEL(cpu_kennel));
    gint dog_indices[2];
    gint compare_result = -1;
    gint override = GPOINTER_TO_INT(user_data);
    
    get_dog_indices(GTK_WIDGET(button), dog_indices);

    /* check for incest; if this is a call from the ok_cancel
       window where the human player has already confirmed
       to proceed, the second argument was 1 */
    if(override != 1 &&
       kinship_of(dogs[dog_indices[0]], dogs[dog_indices[1]]) != 0)
    {
	show_ok_or_cancel_window(GTK_WIDGET(button), " The dogs are near relatives! Are you sure you want to interbreed them? ", 3);

	return 1;
    }
    
    /* if the mating happens within the home kennel,
       we simply do it */
    if(cpu_kennel_id == 1)
    {
	print_message(GTK_WIDGET(button),
		      "The two seem to like each other...");
	/* make pregnant the bitch and set the dog id
	   to 'father' */
	mate(&dogs[dog_indices[0]], &dogs[dog_indices[1]], 0);
	set_save_status(GTK_WIDGET(button), 0);	
    }

    /* else we check whether the cpu-kennel blocks the human player from
       mating */
    else if(options_kennel_block[0][cpu_kennel_id - 2] < 3)
    {
	/* compare dogs */
	compare_result = compare_dogs(dogs[dog_indices[0]], 
				      dogs[dog_indices[1]], 0);
	/* the cpu owner agrees */
	if(compare_result  == 1)
	{
	    print_message(cpu_kennel, "The owner accepts your offer. And the dogs seem to like each other...");
	    
	    mate(&dogs[dog_indices[0]], &dogs[dog_indices[1]], 0);
	    set_save_status(GTK_WIDGET(button), 0);	
	}
	/* maybe the cpu kennel owner accepts although
	   his dog is better if he gets more pups than
	   the human player */
	else if(compare_result == 5)
	{
	    /* if confirmation is on,
	       ask user if he's sure */
	    if( options_confirm[2] == 1 )
	    {
		compare_result = -2;
		show_ok_or_cancel_window(GTK_WIDGET(button), " The owner agrees only if he gets the majority of the pups. Accept? ", 2);
	    }
	    else
	    {
		mate(&dogs[dog_indices[0]], &dogs[dog_indices[1]], 1);
		set_save_status(GTK_WIDGET(button), 0);	
	    }
	}
	else if(compare_result == 2)
	    print_message(GTK_WIDGET(button), "The owner rejects your offer.");
    }
    else
	print_message(GTK_WIDGET(button), "The kennel owner refuses to do business with you. He didn't like your offers in the past, it seems.");

    /* if the mating isn't being handled in the ok_cancel window,
       show kennels and set buttons */
    if(compare_result != -2)
    {
	display_kennel(GTK_WIDGET(button), 1, 0, 10);
	display_kennel(GTK_WIDGET(button), cpu_kennel_id, 1, 10);	
	
	set_buttons(GTK_WIDGET(button));
    }

    return 0;
}

/* display list of puppies */
void
on_show_puppies_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    display_kennel(GTK_WIDGET(button), 21, 0, 0);
    
    set_buttons(GTK_WIDGET(button));
}

/* move a pup from the puppy list to the dog list */
void
on_promote_pup_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
    gint indices[2];

    /* get selected row */
    get_dog_indices(GTK_WIDGET(button), indices);
    
    copy_dogs(dogs[indices[0]], &dogs[first_free_place_in_kennel(0)]);
    dogs[indices[0]].age = -1;
    
    /* redisplay puppy list */
    display_kennel(GTK_WIDGET(button), 21, 0, 0);

    set_save_status(GTK_WIDGET(button), 0);	
    set_buttons(GTK_WIDGET(button));
}

/* sort statistics depending on which column the user
   has clicked */
void
on_dog_info_click_column               (GtkCList        *clist,
                                        gint             column,
                                        gpointer         user_data)
{
    GtkWidget *dog_info_label = lookup_widget(GTK_WIDGET(clist),
					      "dog_info_label");
    GtkWidget *dog_info_label_4 = lookup_widget(GTK_WIDGET(clist),
					      "dog_info_label_4");

    gchar *info_label;
    gint info_label_4 = get_int_from_label(GTK_LABEL(dog_info_label_4));;
    
    gtk_label_get(GTK_LABEL(dog_info_label), &info_label);
    
    if(strncmp(info_label, " Statistics  Dog", 15) == 0)
    {
	if(column > 1 && column < 7)
	    show_statistics(GTK_WIDGET(clist), column * 10, 0, 20);
    }
    else if(strncmp(info_label, " Statistics  Ken", 15) == 0)
    {
	if(column > 1 && column < 7 && info_label_4 == 1)
	    show_statistics(GTK_WIDGET(clist), column * 10, 0, 10);
	else if(column > 1 && column < 5 && info_label_4 == 2)
	    show_statistics(GTK_WIDGET(clist), column * 10, 0, 10);
    }

    set_buttons(GTK_WIDGET(clist));
}

/* show the options window */
void
on_toolbar_options_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *options_window = create_options_window();

    /* set the buttons in the options window corresponding to
       the values of the global option variables */
    set_buttons_options_window(options_window, 0);
    change_popups_active(main_window, 1);
    gtk_widget_show( options_window );    
}

void
on_toolbar_help_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    if(display_help(GTK_WIDGET(button), 0) == 1)
	print_news(GTK_WIDGET(button), "Could not open help file.", 100);
}

void
on_help_back_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    display_help(GTK_WIDGET(button), -1);
}


void
on_help_forward_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    display_help(GTK_WIDGET(button), 1);
}

void
on_toolbar_load_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *file_selection = create_file_selection();
    GtkWidget *file_selection_ok = 
	lookup_widget(file_selection, "file_selection_ok");
    GtkWidget *gui_info = lookup_widget(GTK_WIDGET(button),
					"gui_info");

    /* type determines whether we load a game from within
       a game or at the beginning of a game */
    gint type = GPOINTER_TO_INT(user_data);
    
    if(type == 10)
	gtk_label_set_text(GTK_LABEL(gui_info),
			   "200" );
    else
	gtk_label_set_text(GTK_LABEL(gui_info),
			   "250" );

    if(game_file != NULL)
	gtk_file_selection_set_filename(
	    GTK_FILE_SELECTION(file_selection),
	    game_file);
    
    change_popups_active(main_window, 1);

    if(type != 5)
	gtk_widget_show(file_selection);
    else
	on_file_selection_ok_clicked(GTK_BUTTON(file_selection_ok), 
				     GINT_TO_POINTER(type));
}


void
on_toolbar_save_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *menu_save_as = lookup_widget(GTK_WIDGET(button),
					    "menu_save_as");
    gint quit_after_save = GPOINTER_TO_INT(user_data);

    if(game_file != NULL && options_misc[3] == 1)
    {
	save_game(game_file);
	set_save_status(GTK_WIDGET(button), 1);
	if(quit_after_save == 1)
	    on_toolbar_quit_clicked(button, NULL);
    }
    else
	on_menu_save_as_activate(GTK_MENU_ITEM(menu_save_as),
				 user_data);

    set_buttons(GTK_WIDGET(button));
}

void
on_menu_save_as_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *gui_info = lookup_widget(GTK_WIDGET(menuitem),
					"gui_info");
    GtkWidget *file_selection = 
	create_file_selection();
    GtkWidget *file_selection_ok = lookup_widget(file_selection,
					 "file_selection_ok");
    gint quit_after_save = GPOINTER_TO_INT(user_data);
        
    if(quit_after_save != 1)
	gtk_label_set_text(GTK_LABEL(gui_info),
			    "0" );
    else
	gtk_label_set_text(GTK_LABEL(gui_info),
			   "100" );

    if(game_file != NULL)
	gtk_file_selection_set_filename(
	    GTK_FILE_SELECTION(file_selection),
	    game_file);

    change_popups_active(main_window, 1);
    
    gtk_widget_show(file_selection);
}

gboolean
on_toolbar_quit_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *save_status = lookup_widget(GTK_WIDGET(button),
					   "save_status");
    gint status = get_int_from_label(GTK_LABEL(save_status));
    gint override = GPOINTER_TO_INT(user_data);

    if(override == 1 ||
       options_confirm[4] == 0 || status == 1)
	gtk_main_quit();
    else
	show_ok_or_cancel_window(GTK_WIDGET(button),
				 " Are you sure you want to quit without saving first? ", 50);

    return TRUE;
}

void
on_menu_save_and_quit_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *toolbar_save = lookup_widget(GTK_WIDGET(menuitem),
					    "toolbar_save");

    on_toolbar_save_clicked(GTK_BUTTON(toolbar_save),
			    GINT_TO_POINTER(1));
}

void
on_menu_new_game_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    gint override = GPOINTER_TO_INT(user_data);
    
    if(set_save_status(GTK_WIDGET(menuitem), -1) == 0 &&
		    options_confirm[4] == 1 && override != 1)
    {
	show_ok_or_cancel_window(GTK_WIDGET(menuitem), "Are you sure you want to start a new game and discard your current one?", 10);
    }
    else
    {
	start(GTK_WIDGET(menuitem), 1);
	set_save_status(GTK_WIDGET(menuitem), 0);
    }
}

void
on_menu_help_about_activated           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if(display_help(GTK_WIDGET(menuitem), 50) == 1)
	print_news(GTK_WIDGET(menuitem), "Could not open help file.", 100);
}

void
on_main_buttons_statistics_changed     (GtkOptionMenu   *optionmenu,
                                        gpointer         user_data)
{
    gint selected_item = gtk_option_menu_get_history(optionmenu);

    if(selected_item == 1)
	show_statistics(GTK_WIDGET(optionmenu), 0, 0, 20);
    else if(selected_item == 2)
	show_statistics(GTK_WIDGET(optionmenu), 0, 0, 10);
    else if(selected_item == 3)
	show_champions_race(GTK_WIDGET(optionmenu));
    else if(selected_item == 4)
	show_all_time_records(GTK_WIDGET(optionmenu));
    else if(selected_item == 5)
	show_history(GTK_WIDGET(optionmenu), 1);
    
    set_buttons(GTK_WIDGET(optionmenu));
}
